# Translator.py - code for the Translator product
#
# Copyright (C) 2000 J. David Ibez, Andres Marzal <al028823@alumail.uji.es>
# Copyright (C) 2000 Federico Di Gregorio  <fog@mixadlive.com>
# Copyright (C) 2000 MIXAD LIVE S.r.l.  [http://www.mixadlive.com]
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by the
# Free Software Foundation; either version 2, or (at your option) any later
# version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTIBILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# -*- Mode: pyhton -*-

__doc__ = """Translator"""
__version__ = '0.6'
__author__ = """J. David Ibez <al028823@alumail.uji.es>
                Federico Di Gregorio <fog@mixadlive.com>"""


import sys, string
from string import split, strip, join

# zope specific imports
from Globals import HTMLFile, MessageDialog, PersistentMapping
from OFS.Folder import Folder
from OFS.PropertyManager import PropertyManager
from Shared.xmlio import XMLio
from DocumentTemplate.DT_Util import *
import OFS, Acquisition


TRANSLATE_PERMISSION = 'Translate'


#### DTML manager and object creator function

manage_addTranslatorForm = HTMLFile('translatorAdd', globals())

def manage_addTranslator(self, title='', lang='en', cookieName='',
                         createFolders=0, REQUEST=None):
    """Create and add a new Translator."""
    o = Translator('translator', title, lang, cookieName)
    self._setObject('translator', o)

    if createFolders:
        u = REQUEST['AUTHENTICATED_USER']    
        if (u is not None) and not u.has_permission('Add Folders', self):
            raise 'Unauthorized', ('You are not authorized to add Folders.')
        for l in split(lang):
            l = strip(l)
            if len(l) > 0:
                self.manage_addFolder(id=l, title='',
                                      createPublic=0, createUserF=0,
                                      REQUEST=REQUEST)
       
    # return the management interface for the calling object
    if REQUEST is not None:
        return self.manage_main(self, REQUEST, update_menu=1)


#### little utility function to log debug information

def _trace(msg):
    sys.stderr.write('translator: %s\n' % msg)


#### the class

class Translator(Folder, XMLio):
    """Translator class."""

    meta_type = 'Translator'

    manage_options = Folder.manage_options[:2] + (
        {'label' : 'Edit', 'action' : 'manage_editForm'},
        {'label' : 'Translate', 'action' : 'manage_translateForm'},
        {'label':'XML', 'action':'XMLForm'}
        ) + Folder.manage_options[2:-2]

    __ac_permissions__ = Folder.__ac_permissions__ + (
        (TRANSLATE_PERMISSION, ['add', 'delete', 'addTranslation']),)

    # managing pages
    manage_editForm = HTMLFile('translatorEdit', globals())
    manage_translateForm = HTMLFile('translatorTranslate', globals())
    manage_translateItemForm = HTMLFile('translatorTranslateItem', globals())
    manage_renameForm = HTMLFile('translatorRenameItem', globals())
    
    def __init__(self, id, title, lang, cookieName):
        """Initialize the object."""
        self.id = id
        self.title = title
        self.default_languages = split(lang)
        self.cookie = cookieName
        self.vocabulary = PersistentMapping()

    # here are the juicy functions
    def __get_languages__(self, REQUEST=None):
        if REQUEST:
	    if REQUEST.has_key(self.cookie):
	        lang = string.split(REQUEST[self.cookie])
            elif REQUEST.cookies.has_key(self.cookie):
                lang = string.split(REQUEST.cookies[self.cookie])
            elif REQUEST.has_key('HTTP_ACCEPT_LANGUAGE'):
                s = string.replace(REQUEST['HTTP_ACCEPT_LANGUAGE'], ' ', '')
                lang = string.split(s, ',')
        else:
            lang = []
        #_trace('__get_languages__(): lang = %s' % `lang`)
        lang = lang + self.default_languages
        if lang: return lang
        else: return ['en']


    def __call__(self, key, REQUEST=None, PARENTS=None):
        """Return the translation of string, if available.

           The algoritm here is: first try to find the translation in the
           vocabulary for this object, then try to find it as property of
           the objects accesible (acquisition!) from this one. Eventually
           the standard acquisition rules are used."""

        if REQUEST:
            lang = self.__get_languages__(REQUEST)
        else:
            lang = self.__get_languages__(getattr(self, 'REQUEST'))

        if self.vocabulary.has_key(key):
            for l in lang:
                if self.vocabulary[key].has_key(l):
                    return self.vocabulary[key][l]

        # not found in the dictionary? lets try the iso folders
        for l in lang:
            try:
                folder = getattr(self, l)
                return getattr(folder, key)
            except AttributeError:
                # a key error at this point is not fatal, because
                # one of the next languages in lang can do the job,
                # to raise we wait until we exit the loop
                pass

        # still not found? let's try acquisition
        if PARENTS is None:
            try:
                next = self.aq_parent.aq_parent.translator
            except:
                next = None
        elif len(PARENTS) > 0:
            next = PARENTS[0].translator
            PARENTS = PARENTS[1:]
        else:
            next = None

        if next:
            return next(key, REQUEST, PARENTS)
        else:
            return getattr(self, key)


    def getDefaultLanguages(self):
        """Get default languages and make blank-separated list."""
        return join(self.default_languages)

    def getLanguages(self, REQUEST=None):
        """Return a list of preferred languages."""
        if REQUEST:
            return self.__get_languages__(REQUEST)
        else:
            return self.__get_languages__(getattr(self, 'REQUEST'))

    def getLanguage(self, REQUEST=None):
        """Return preferred language."""
        return self.getLanguages(REQUEST)[0]

    def getPreferenceCookie(self):
        """Get language cookie."""
        return self.cookie


    def manage_edit(self, lang, cookieName, REQUEST=None):
        """Edit inner object properties."""
        self.cookie = cookieName
        self.default_languages = split(lang)
        #_trace('manage_edit(): cookie changed to %s' % self.cookie)
        if REQUEST is not None:
            return MessageDialog(
                title = 'Edited',
                message = 'Properties for %s changed.' % self.id,
                action = './manage_editForm')            


    def search(self, filt):
        """Filter the keys in the vocabulary dictionary."""
        if filt == '':
            return self.vocabulary.items()
        else:
            try:
                return [(filt, self.vocabulary[filt])]
            except:
                return filter(lambda x, s=filt: string.find(x[0],s) != -1,
                              self.vocabulary.items())


    def add(self, string, REQUEST=None):
        """Add a new localized property."""
        keys = split(string)
        for key in keys:
            key = strip(key)
            if len(key) > 0:
                self.vocabulary[key] = PersistentMapping()
                for l in self.default_languages:
                    self.vocabulary[key][l] = ''
        if REQUEST is not None:
            REQUEST['string'] = ''
            return self.manage_translateForm(self, REQUEST)


    def delete_or_rename(self, items, delete=0, rename=0, REQUEST=None):
        """Delete translated properties from dictionary."""
        if rename and len(items) > 0:
            return self.manage_renameForm(self, REQUEST, item=items[0])
        
        elif delete:
            for item in items:
                del self.vocabulary[item]

        if REQUEST is not None:
            return self.manage_translateForm(self, REQUEST)

    def rename(self, item, name, REQUEST=None):
        """Rename the given item."""
        if len(name) > 0:
            self.vocabulary[name] = self.vocabulary[item]
            del self.vocabulary[item]
        if REQUEST is not None:
            return self.manage_translateForm(self, REQUEST)


    def delTranslations(self, entry, items, REQUEST=None):
        """Delete translations from items."""
        for item in items:
            del self.vocabulary[entry][item]
        if REQUEST is not None:
            return self.manage_translateForm(self, REQUEST)


    def addTranslation(self, entry, language, translation, REQUEST):
        """Add a new translation to a localized property."""
        self.vocabulary[entry][language] = translation
        if REQUEST is not None:
            return self.manage_translateItemForm(self, REQUEST)


    def translate(self, key, REQUEST=None, PARENTS=None):
        """This should be analogue to the dtml-translate (or dtml-tvar) tag."""
        return self.__call__(key, method, REQUEST, PARENTS)


    def localize(self, key, **kw):
        """This should be analogue to the dtml-localize (or dtml-lvar) tag."""
        ns = self
        if kw.has_key('namespace'): ns = kw['namespace']
        if kw.has_key('ns'): ns = kw['ns']
        v = apply(self, (key,), kw)
        return str(render(ns, v))


    def localized_url(self, key, **kw):
        """Returns the url of the given object."""
        v = apply(self, (key,), kw)
        return v.absolute_url()
    
        
    # XML support!
    xml_name = 'translator'
    xml_attributes = ('id', 'title')

    def xml_get_elements(self):
        elements = []
        for string, translations in self.vocabulary.items():
            t = []
            for language, translation in translations.items():
                t.append('<translation language="%s" value="%s" />'
                         % (language, translation))
                elements.append(('<entry id="%s">' % string) +
                                join(t,'\n') + '</entry>')
        return elements



